//
// Zaks KeyBoard
//    ver 1.0.0b (still under construction)
//
//    by ZakLab 2024
//
//    for Arduino Pro Micro with Arduino Leonardo mode
//
//    This code is free for use.  But copyright by ZakLab Zak Sawa.
//
//
// Release record
//    ver 1.0.0b :: first release.  Key access is OK.  LED is not implemented.  No chattering fix.
//



// includes
#include <Keyboard.h>


// defines
#define kMaxSelector 10
static const uint8_t kSelectPort[kMaxSelector] = { 8, 9, 10, 14, 15, 16, 18, 19, 20, 21 };


// global vars
static const uint8_t kKeyMatrix[kMaxSelector][8] = { { 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i' },
                                                     { 'o', 'p', '[', ']', 'a', 's', 'd', 'f' },
                                                     { 'g', 'h', 'j', 'k', 'l', ';', 0x27, 'z' },
                                                     { 'x', 'c', 'v', 'b', 'n', 'm', ',', '.' },
                                                     { '/', '1', '2', '3', '4', '5', '6', '7' },
                                                     { '8', '9', '0', '-', '=', 0x5C, '`', ' ' },
                                                     { KEY_ESC, KEY_TAB, KEY_LEFT_CTRL, KEY_LEFT_SHIFT, KEY_BACKSPACE, KEY_RETURN, KEY_RIGHT_SHIFT, 0 },
                                                     { KEY_LEFT_ALT, KEY_LEFT_GUI, KEY_RIGHT_GUI, KEY_RIGHT_ALT, KEY_UP_ARROW, KEY_LEFT_ARROW, KEY_DOWN_ARROW, KEY_RIGHT_ARROW },
                                                     { KEY_F1, KEY_F2, KEY_F3, KEY_F4, KEY_F5, KEY_F6, KEY_F7, KEY_F8 },
                                                     { KEY_F9, KEY_F10, KEY_F11, KEY_F12, 0, 0, 0, 0 } };

#if 0
static const uint8_t kKeyMatrixShift[kMaxSelector][8] = { { 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I' },
                                                          { 'O', 'P', '{', '}', 'A', 'S', 'D', 'F' },
                                                          { 'G', 'H', 'J', 'K', 'L', ':', '"', 'Z' },
                                                          { 'X', 'C', 'V', 'B', 'N', 'M', '<', '>' },
                                                          { '?', '!', '@', '#', '$', '%', '^', '&' },
                                                          { '*', '(', ')', '_', '+', '|', '~', ' ' },
                                                          { KEY_ESC, KEY_TAB, KEY_LEFT_CTRL, KEY_LEFT_SHIFT, KEY_DELETE, KEY_RETURN, KEY_RIGHT_SHIFT, 0 },
                                                          { KEY_LEFT_ALT, KEY_LEFT_GUI, KEY_RIGHT_GUI, KEY_RIGHT_ALT, KEY_UP_ARROW, KEY_LEFT_ARROW, KEY_DOWN_ARROW, KEY_RIGHT_ARROW },
                                                          { KEY_F1, KEY_F2, KEY_F3, KEY_F4, KEY_F5, KEY_F6, KEY_F7, KEY_F8 },
                                                          { KEY_F9, KEY_F10, KEY_F11, KEY_F12, 0, 0, 0, 0 } };
#endif

static uint8_t gKeyPress[kMaxSelector] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static uint8_t gLED_VRAM[kMaxSelector] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };


// functions
void setup() {
  uint8_t counter;

  // setup all Selector port
  for (counter = 0; counter < kMaxSelector; counter++) {
    pinMode(kSelectPort[counter], INPUT);
    digitalWrite(kSelectPort[counter], LOW);
  }

  // setup all Data port
  for (counter = 0; counter < 8; counter++)
    pinMode(counter, INPUT);

  // start keyboard
  Keyboard.begin();
  delay(1000);
}


static void checkData(uint16_t selector,
                      uint16_t dataBit) {
  if (digitalRead(dataBit) == HIGH) {
    if ((gKeyPress[selector] & (1 << dataBit)) == 0)
      Keyboard.press(kKeyMatrix[selector][dataBit]);

    gKeyPress[selector] |= (1 << dataBit);
  } else {
    if ((gKeyPress[selector] & (1 << dataBit)) != 0)
      Keyboard.release(kKeyMatrix[selector][dataBit]);

    gKeyPress[selector] &= ~(1 << dataBit);
  }

  // Keyboard.press();
  // Keyboard.release();
  // Keyboard.releaseAll();
  // Keyboard.write();
  // Keyboard.print();
  // Keyboard.println();
}


static void updateLED(uint16_t selector,
                      uint16_t dataBit) {
  if ((gLED_VRAM[selector] & (1 << dataBit)) != 0) {
    pinMode(dataBit, OUTPUT);
    digitalWrite(dataBit, HIGH);
    //    delay(1);
    digitalWrite(dataBit, LOW);
    pinMode(dataBit, INPUT);
  }
}


void loop() {
  static int16_t sCurrentSelector = 0;

  // setup current key scan
  pinMode(kSelectPort[sCurrentSelector], OUTPUT);
  digitalWrite(kSelectPort[sCurrentSelector], HIGH);
  // delay(1);

  // check each data
  checkData(sCurrentSelector, 0);
  checkData(sCurrentSelector, 1);
  checkData(sCurrentSelector, 2);
  checkData(sCurrentSelector, 3);
  checkData(sCurrentSelector, 4);
  checkData(sCurrentSelector, 5);
  checkData(sCurrentSelector, 6);
  checkData(sCurrentSelector, 7);

  // setup current LED scan
  digitalWrite(kSelectPort[sCurrentSelector], LOW);

#if 0
  // set LED on and off
  updateLED(sCurrentSelector, 0);
  updateLED(sCurrentSelector, 1);
  updateLED(sCurrentSelector, 2);
  updateLED(sCurrentSelector, 3);
  updateLED(sCurrentSelector, 4);
  updateLED(sCurrentSelector, 5);
  updateLED(sCurrentSelector, 6);
  updateLED(sCurrentSelector, 7);
#endif

  // finish current scan
  pinMode(kSelectPort[sCurrentSelector], INPUT);

  sCurrentSelector++;
  if (sCurrentSelector >= kMaxSelector)
    sCurrentSelector = 0;
}
